@@ -1,6 +1,9 @@
 package Terminal::Control;
+# This package provides the basis for improving and studying Terminal Window
+# control. Essentially, the Perl header 'sys/ioctl.ph', the SHELL command stty
+# and ASCII escape sequences are used.
 
-# Load the Perl pragmas.
+# Load the basic Perl pragmas.
 use 5.030000;
 use strict;
 use warnings;
@@ -8,8 +11,49 @@ use warnings;
 # Load the Perl pragma Exporter.
 use Exporter;
 
-# Load Perl modules.
+# Load the neccessary Perl modules.
 use bytes;
+use Try::Catch;
+use Time::HiRes qw(ualarm);
+
+# Set the package version. 
+our $VERSION = '0.05';
+
+# Base class of this module.
+our @ISA = qw(Exporter);
+
+# Exporting the implemented subroutines.
+our @EXPORT = qw(
+    reset_screen
+    clear_screen
+    reset_terminal  
+    clear_terminal  
+    reset
+    clear
+    winsize
+    chars
+    pixels
+    get_cursor_position
+    set_cursor_position
+    echo_on
+    echo_off 
+    cursor_on
+    cursor_off
+    winsize_pixels
+    screen_size_pixels
+    screen_size_chars
+    window_size_chars
+    window_size_pixels
+    ctlseqs_request
+);
+
+# Set the global variables.
+our($ESC, $CSI, $OSC);
+
+# Set some constants.
+$ESC = "\033";  # Escape
+$CSI = "\033["; # Control Sequence Introducer
+$OSC = "\033]"; # Operating System Command
 
 # Declare global variable
 my $is_eval;
@@ -17,7 +61,7 @@ my $is_eval;
 # BEGIN block.
 BEGIN {
     # Try to load sys/ioctl.ph.
-    $is_eval = ((eval "require 'sys/ioctl.ph'") ? 1 : 0);
+    $is_eval = ((eval "require 'sys/ioctl1.ph'") ? 1 : 0);
     if ($is_eval ne 1) {
         # Print message into the terminal window.
         print "sys/ioctl.ph is missing\n";
@@ -27,45 +71,13 @@ BEGIN {
         # Assign winsize to winsize_ioctl.
         *winsize = \&winsize_ioctl;
     };
-    # Define subroutine aliases.
+    # Define some subroutine aliases.
     *reset_terminal = \&reset_screen;
     *clear_terminal = \&clear_screen;
     *reset = \&reset_screen;
     *clear = \&clear_screen;
 };
 
-# Set the package version. 
-our $VERSION = '0.04';
-
-# Base class of this module.
-our @ISA = qw(Exporter);
-
-# Exporting the implemented subroutines.
-our @EXPORT = qw(
-                 reset_screen
-                 clear_screen
-                 reset_terminal  
-                 clear_terminal  
-                 reset
-                 clear
-                 winsize
-                 chars
-                 pixels
-                 get_cursor_position
-                 set_cursor_position
-                 echo_on
-                 echo_off 
-                 cursor_on
-                 cursor_off
-);
-
-# Set the global variables.
-our($ESC, $CSI);
-
-# Set the Control Sequence Introducer (CSI).
-$CSI = "\033[";
-$ESC = "\033";
-
 #------------------------------------------------------------------------------# 
 # Subroutine clear_screen                                                      #
 #                                                                              #
@@ -116,7 +128,7 @@ sub cursor_on {
 # stty for reading from tty and writing to tty. Redirect stderr to stdout.     #
 #------------------------------------------------------------------------------# 
 sub cursor_off {
-    # Hides the cursor.
+    # Hide the cursor.
     print "${CSI}?25l";
 };
 
@@ -165,9 +177,9 @@ sub get_cursor_position {
     # Restore the settings of the terminal window.
     system "stty -cbreak echo </dev/tty >/dev/tty 2>&1";
     # Get rows and cols.
-    my ($y, $x) = $seq =~ m/(\d+)\;(\d+)/;
+    my ($y, $x) = $seq =~ /(\d+)\;(\d+)/;
     # Return rows and cols.
-    return ($y, $x)
+    return ($y, $x);
 };
 
 #------------------------------------------------------------------------------# 
@@ -184,6 +196,151 @@ sub set_cursor_position {
     print "${CSI}${n};${m}H";
 };
 
+#------------------------------------------------------------------------------# 
+# Subroutine ctlseqs_request                                                   #
+#                                                                              #
+# Description:                                                                 #
+# Get the response from escape sequence.                                       #
+#------------------------------------------------------------------------------# 
+sub ctlseqs_request {
+    # Assign the subroutine arguments to the local variables.
+    my ($Ps, $user_timeout) = @_;
+    # Define the signal handler.
+    local $SIG{ALRM} = sub {die "timeout"};
+    # Initialise the local variables.
+    my ($width, $height) = undef;
+    my $buf = '';
+    my $chr = '';
+    my $timeout = 1_000_000;
+    # Change the timeout.
+    $timeout = (defined $user_timeout) ? $user_timeout : $timeout;
+    # Change the settings of the terminal window.
+    system "stty cbreak -echo </dev/tty >/dev/tty 2>&1";
+    # Print the escape sequence.
+    print STDOUT "${CSI}${Ps}t";
+    # Non blocking reading from STDIN.
+    eval {
+        # Invoke an alarm.
+        ualarm $timeout;
+        # Read chars from STDIN. 
+        while (($chr = getc(STDIN)) ne "") {
+            $buf .= $chr;
+        };
+        # Stop the alarm.
+        ualarm 0;
+    };
+    if ($buf eq '') {
+        # Set width and height.
+        ($width, $height) = (-1, -1);
+    } else {
+        # Get width and height.
+        my $re = qr/\d+;(\d+)\;(\d+)/;
+        ($width, $height) = ($buf =~ $re) ? ($buf =~ $re) : (-2, -2);
+    };
+    # Restore the settings of the terminal window.
+    system "stty -cbreak echo </dev/tty >/dev/tty 2>&1";
+    # Return width and height.
+    return ($width, $height);
+};
+
+#------------------------------------------------------------------------------# 
+# Subroutine window_size_pixels                                                #
+#                                                                              #
+# Description:                                                                 #
+# Xterm Control Sequences                                                      #
+# Request: CSI Ps t                                                            #
+# Response: Ps = 14 -> Reports xterm window size in pixels as                  #
+#                      CSI 4 ; width ; height t                                #
+#------------------------------------------------------------------------------# 
+sub window_size_pixels {
+    # Set the timeout.
+    my $timeout = $_[0];
+    # Get width and height.
+    my ($x, $y) = ctlseqs_request("14", $timeout);
+    # Return width and height.
+    return ($x, $y);
+};
+
+#------------------------------------------------------------------------------# 
+# Subroutine screen_size_pixels                                                #
+#                                                                              #
+# Description:                                                                 #
+# Xterm Control Sequences                                                      #
+# Request: CSI Ps t                                                            #
+# Response: Ps = 15 -> Reports xterm screen size in pixels as                  #
+#                      CSI 5 ; width ; height t                                #
+#------------------------------------------------------------------------------# 
+sub screen_size_pixels {
+    # Set the timeout.
+    my $timeout = $_[0];
+    # Get width and height.
+    my ($x, $y) = ctlseqs_request("15", $timeout);
+    # Return width and height.
+    return ($x, $y);
+};
+
+#------------------------------------------------------------------------------# 
+# Subroutine window_size_chars                                                 #
+#                                                                              #
+# Description:                                                                 #
+# Xterm Control Sequences                                                      #
+# Request: CSI Ps t                                                            #
+# Response: Ps = 18 -> Reports xterm window size in chars as                   #
+#                      CSI 8 ; width ; height t                                #
+#------------------------------------------------------------------------------# 
+sub window_size_chars {
+    # Set the timeout.
+    my $timeout = $_[0];
+    # Get width and height.
+    my ($x, $y) = ctlseqs_request("18", $timeout);
+    # Return width and height.
+    return ($x, $y);
+};
+
+#------------------------------------------------------------------------------# 
+# Subroutine screen_size_chars                                                 #
+#                                                                              #
+# Description:                                                                 #
+# Xterm Control Sequences                                                      #
+# Request: CSI Ps t                                                            #
+# Response: Ps = 19 -> Reports xterm window size in chars as                   #
+#                      CSI 9 ; width ; height t                                #
+#------------------------------------------------------------------------------# 
+sub screen_size_chars {
+    # Set the timeout.
+    my $timeout = $_[0];
+    # Get width and height.
+    my ($height, $width) = ctlseqs_request("19", $timeout);
+    # Return width and height.
+    return ($height, $width);
+};
+
+#------------------------------------------------------------------------------# 
+# Subroutine winsizwe_pixels                                                   #
+#                                                                              #
+# Description:                                                                 #
+# Get winsize in pixels.                                                       #
+#------------------------------------------------------------------------------# 
+sub winsize_pixels {
+    # Initialise the local variables.
+    my $seq = '';
+    my $chr = '';
+    # Change the settings of the terminal window.
+    system "stty cbreak -echo </dev/tty >/dev/tty 2>&1";
+    # Print escape sequence.
+    print STDOUT "${CSI}14t";
+    # Read chars von STDIN.
+    while (($chr = getc(STDIN)) ne "t") {
+        $seq .= $chr;
+    };
+    # Restore the settings of the terminal window.
+    system "stty -cbreak echo </dev/tty >/dev/tty 2>&1";
+    # Get rows and cols.
+    my ($x, $y) = $seq =~ m/(\d+)\;(\d+)$/;
+    # Return rows and cols.
+    return ($x, $y)
+};
+
 #------------------------------------------------------------------------------# 
 # Subroutine winsize                                                           #
 #                                                                              #
@@ -191,12 +348,20 @@ sub set_cursor_position {
 # Get winsize using system ioctl call.                                         #
 #------------------------------------------------------------------------------# 
 sub winsize_ioctl {
-    # Initialise the variable $winsize. 
+    # Initialise the local variables.
+    my ($rows, $cols, $xpix, $ypix) = (undef, undef, undef, undef);
     my $winsize = "";
-    # Check the ioctl call of TIOCGWINSZ.
-    my ($rows, $cols, $xpix, $ypix) = ((ioctl(STDOUT, TIOCGWINSZ(), $winsize)) ?
-                                       (unpack 'S4', $winsize) :
-                                       (map {$_ * 0} (1..4)));
+    # Try to get the winsize.
+    try {
+        # Check the ioctl call of TIOCGWINSZ.
+        ($rows, $cols, $xpix, $ypix) = (
+            (ioctl(STDOUT, TIOCGWINSZ(), $winsize)) ?
+            (unpack 'S4', $winsize) :
+            (map {$_ * 0} (1..4))
+            );
+    } catch {
+        ($rows, $cols, $xpix, $ypix) = (-1, -1, -1, -1);
+    };
     # Return rows, cols and xpix, ypix.
     return ($rows, $cols, $xpix, $ypix);
 };
@@ -211,10 +376,10 @@ sub winsize_stty {
     # Initialise the local variable.
     my $winsize = "";
     # Get winsize from stty.
-    $winsize = `stty size`;
-    my ($x, $y) = $winsize =~ m/(\d+) (\d+)/s;
+    $winsize = qx/stty size/;
+    my ($y, $x) = $winsize =~ m/(\d+) (\d+)/s;
     # Create list with rows, cols, xpix and ypix. 
-    my ($rows, $cols, $xpix, $ypix) = ($x, $y, -1, -1);
+    my ($rows, $cols, $xpix, $ypix) = ($y, $x, -1, -1);
     # Return rows, cols and xpix, ypix.
     return ($rows, $cols, $xpix, $ypix);
 };
@@ -260,7 +425,7 @@ __END__
 
 =head1 NAME
 
-Terminal::Control - Perl extension for terminal control
+Terminal::Control - Perl extension for the terminal window control
 
 =head1 SYNOPSIS
 
@@ -289,20 +454,41 @@ Terminal::Control - Perl extension for terminal control
   printf ("%s\n%s\n", $y, $x);
 
   # Set cursor position.
-  my $rows = 20; 
-  my $cols = 80; 
-  set_cursor_position(20, 80);
-
-=head1 Aliases
-
-  reset_terminal <= reset_screen;
-  clear_terminal <= clear_screen;
-  reset          <= reset_screen;
-  clear          <= clear_screen;
+  my $y = 20; 
+  my $x = 80; 
+  set_cursor_position($y, $x);
 
 =head1 Requirement
 
-The Perl header 'sys/ioctl.ph' is required for ioctl and TIOCGWINSZ.
+The Perl header 'sys/ioctl.ph' is required for the ioctl call of the function
+TIOCGWINSZ. The equivalent C/C++ header is 'sys/ioctl.h'. The Perl command h2ph
+converts '.h' C/C++ header files to '.ph' Perl header files.
+
+In contrast to modules in general, the module installation process cannot be told
+to create a sys/ioctl.ph. This is necessary manually via the h2ph command.
+
+To prevent tests within CPAN from failing due to a missing sys/ioctl.ph, a fallback
+solution based on the BASH command stty is programmed.   
+
+=head1 Motivation
+
+The idea for the necessity of the module arises from the fact that especially 
+the system call system("reset") of the BASH command reset is noticeably slow.
+The BASH command reset is slow, the system call system("reset") is slower. By
+using so-called ASCII escape sequences, a significant acceleration can be
+achieved. The logical consequence is the programming of a Perl command that
+replaces the system call. A simple method is the best way to realise this. There
+is no need to implement a special Class to achive this.  
+
+=head1 Benchmark
+
+Exemplary comparison comparison of time of execution:
+
+system("reset")  =>  1.026892 Seconds = 1026,892 Milliseconds = 1026892 Microseconds
+  
+reset_screen()   =>  0.000076 Seconds =    0,076 Milliseconds = 76 Microseconds
+
+This one example already shows a significant advantage of the new routine.
 
 =head1 DESCRIPTION
 
@@ -336,6 +522,15 @@ The following methods have been implemented so far:
 
 =back
 
+=head1 Aliases
+
+clear_screen and reset_screen can also be used with this aliases:
+
+  reset_terminal  <=  reset_screen
+  clear_terminal  <=  clear_screen
+  reset           <=  reset_screen
+  clear           <=  clear_screen
+
 =head2 Method description
 
 The method clear_screen is clearing the terminal. This is similar to the system call system('clear').
@@ -347,6 +542,7 @@ and pixels extract chars (rows and cols) and pixels (xpix and ypix.)
 The method get_cursor_position() gets the current cursor position in the terminal window. The method
 set_cursor_position() sets the cursor position in the terminal window.
  
+echo_on and echo_off turnes echo of commands on or off. cursor_on and cursor_off shows or hides the cursor.
 
 =head1 Programmable background
 
